@@ -17,18 +17,39 @@
 
 namespace Mercator {
 
+const std::string DepthShader::key_waterLevel("waterLevel");
+const std::string DepthShader::key_murkyDepth("murkyDepth");
+
+const float DepthShader::default_waterLevel = 0.f;
+const float DepthShader::default_murkyDepth = -64.f;
+
+
 DepthShader::DepthShader(float waterLevel, float murkyDepth) : 
-              m_waterLevel(waterLevel), m_murkyDepth(murkyDepth)
+             m_waterLevel(waterLevel), m_murkyDepth(murkyDepth)
 {
 }
 
+DepthShader::DepthShader(const Parameters & params) :
+             m_waterLevel(default_waterLevel), m_murkyDepth(default_murkyDepth)
+{
+    Parameters::const_iterator I = params.find(key_waterLevel);
+    Parameters::const_iterator Iend = params.end();
+    if (I != Iend) {
+        m_waterLevel = I->second;
+    }
+    I = params.find(key_murkyDepth);
+    if (I != Iend) {
+        m_murkyDepth = I->second;
+    }
+}
+
 DepthShader::~DepthShader()
 {
 }
 
-bool DepthShader::checkIntersect(Surface & s) const
+bool DepthShader::checkIntersect(const Segment & s) const
 {
-    if (s.m_segment.getMin() < m_waterLevel) {
+    if (s.getMin() < m_waterLevel) {
         return true;
     } else {
         return false;
