@@ -13,11 +13,15 @@ FillShader::FillShader()
 {
 }
 
+FillShader::FillShader(const Parameters & params)
+{
+}
+
 FillShader::~FillShader()
 {
 }
 
-bool FillShader::checkIntersect(Surface & s) const
+bool FillShader::checkIntersect(const Segment &) const
 {
     return true;
 }
